@@ -3,8 +3,6 @@
 use lib '../lib';
 use strict;
 use warnings;
-use Coro;
-use Coro::Event;
 use URI::Escape;
 
 =head1 Summary
@@ -19,8 +17,7 @@ code. We even implement our own 'prompt'...
 use Continuity;
 my $server = new Continuity;
 
-Event::loop();
-
+$server->loop;
 
 # Ask a question and keep asking until they answer
 sub prompt {
@@ -42,6 +39,7 @@ sub main {
 
   # After we're done with that we enter a loop. Forever.
   while(1) {
+    print STDERR "My request is $request\n";
     print STDERR "Just about to suspend...\n";
     my $add = $request->next->param('add');
     print STDERR "*** Just grabed next param\n";
